use super::ApiError;
use crate::definitions::vpn::{WireguardInterface, WireguardPeer};
use crate::state::RpcState;
use crate::{
    create_thing, delete_thing_by_name, get_thing_by_name, list_things, update_thing_by_name,
};
use jsonrpsee::types::Params;
use jsonrpsee::RpcModule;

pub fn register_methods(module: &mut RpcModule<RpcState>) {
    module
        .register_method("vpn.wireguard.status", wireguard_status)
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.interfaces.get",
            get_thing_by_name!(vpn.wireguard.interfaces),
        )
        .unwrap();

    module
        .register_method::<Result<Vec<WireguardInterface>, ApiError>, _>(
            "vpn.wireguard.interfaces.list",
            list_things!(vpn.wireguard.interfaces),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.interfaces.create",
            create_thing!(vpn.wireguard.interfaces, WireguardInterface),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.interfaces.update",
            update_thing_by_name!(vpn.wireguard.interfaces, WireguardInterface),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.interfaces.delete",
            delete_thing_by_name!(vpn.wireguard.interfaces),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.peers.get",
            get_thing_by_name!(vpn.wireguard.peers),
        )
        .unwrap();

    module
        .register_method::<Result<Vec<WireguardPeer>, ApiError>, _>(
            "vpn.wireguard.peers.list",
            list_things!(vpn.wireguard.peers),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.peers.create",
            create_thing!(vpn.wireguard.peers, WireguardPeer),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.peers.update",
            update_thing_by_name!(vpn.wireguard.peers, WireguardPeer),
        )
        .unwrap();

    module
        .register_method(
            "vpn.wireguard.peers.delete",
            delete_thing_by_name!(vpn.wireguard.peers),
        )
        .unwrap();
}

pub fn wireguard_status(_: Params, _: &RpcState) -> Result<String, ApiError> {
    Ok("ok".to_string())
}
